@@ -1,7 +1,6 @@
 #!/usr/bin/env pytest
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test default implementation of GDALRasterBand::IRasterIO
@@ -10,23 +9,7 @@
 ###############################################################################
 # Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import math
@@ -38,14 +21,6 @@ import pytest
 
 from osgeo import gdal
 
-
-###############################################################################
-@pytest.fixture(autouse=True, scope="module")
-def module_disable_exceptions():
-    with gdaltest.disable_exceptions():
-        yield
-
-
 ###############################################################################
 # Test writing a 1x1 buffer to a 10x6 raster and read it back
 
@@ -233,6 +208,7 @@ def test_rasterio_4():
 # Test error cases of ReadRaster()
 
 
+@gdaltest.disable_exceptions()
 def test_rasterio_5():
 
     ds = gdal.Open("data/byte.tif")
@@ -300,6 +276,7 @@ def test_rasterio_5():
 # Test error cases of WriteRaster()
 
 
+@gdaltest.disable_exceptions()
 def test_rasterio_6():
 
     ds = gdal.GetDriverByName("MEM").Create("", 2, 2)
@@ -732,6 +709,7 @@ def test_rasterio_9():
 
 def test_rasterio_overview_subpixel_resampling():
 
+    gdaltest.importorskip_gdal_array()
     numpy = pytest.importorskip("numpy")
 
     temp_path = "/vsimem/rasterio_ovr.tif"
@@ -773,6 +751,7 @@ def test_rasterio_overview_subpixel_resampling():
 # Test error when getting a block
 
 
+@gdaltest.disable_exceptions()
 def test_rasterio_10():
     ds = gdal.Open("data/byte_truncated.tif")
 
@@ -796,6 +775,8 @@ def test_rasterio_10():
 
 
 def test_rasterio_11():
+
+    gdaltest.importorskip_gdal_array()
     numpy = pytest.importorskip("numpy")
 
     mem_ds = gdal.GetDriverByName("MEM").Create("", 4, 3)
@@ -832,6 +813,8 @@ def rasterio_12_progress_callback(pct, message, user_data):
 
 
 def test_rasterio_12():
+
+    gdaltest.importorskip_gdal_array()
     numpy = pytest.importorskip("numpy")
 
     mem_ds = gdal.GetDriverByName("MEM").Create("", 4, 3, 4)
@@ -900,6 +883,8 @@ def test_rasterio_12():
     ],
 )
 def test_rasterio_13(dt):
+
+    gdaltest.importorskip_gdal_array()
     numpy = pytest.importorskip("numpy")
 
     dt = gdal.GetDataTypeByName(dt)
@@ -972,7 +957,7 @@ def test_rasterio_13(dt):
 @pytest.mark.parametrize("use_nan", [True, False])
 def test_rasterio_nearest_or_mode(dt, resample_alg, use_nan):
     numpy = pytest.importorskip("numpy")
-    gdal_array = pytest.importorskip("osgeo.gdal_array")
+    gdal_array = gdaltest.importorskip_gdal_array()
 
     dt = gdal.GetDataTypeByName(dt)
     mem_ds = gdal.GetDriverByName("MEM").Create("", 4, 4, 1, dt)
@@ -1228,6 +1213,8 @@ cellsize     0
 
 
 def test_rasterio_nodata():
+
+    gdaltest.importorskip_gdal_array()
     pytest.importorskip("numpy")
 
     ndv = 123
@@ -1435,6 +1422,8 @@ nodata_value 0
 
 
 def test_rasterio_dataset_readarray_cint16():
+
+    gdaltest.importorskip_gdal_array()
     numpy = pytest.importorskip("numpy")
 
     mem_ds = gdal.GetDriverByName("MEM").Create("", 1, 1, 2, gdal.GDT_CInt16)
@@ -1447,6 +1436,7 @@ def test_rasterio_dataset_readarray_cint16():
     assert got[1] == numpy.array([[3 + 4j]])
 
 
+@gdaltest.disable_exceptions()
 def test_rasterio_rasterband_write_on_readonly():
 
     ds = gdal.Open("data/byte.tif")
@@ -1456,6 +1446,7 @@ def test_rasterio_rasterband_write_on_readonly():
     assert err != 0
 
 
+@gdaltest.disable_exceptions()
 def test_rasterio_dataset_write_on_readonly():
 
     ds = gdal.Open("data/byte.tif")
@@ -1508,6 +1499,8 @@ def test_rasterio_floating_point_window_no_resampling():
 
 def test_rasterio_floating_point_window_no_resampling_numpy():
     # Same as above but using ReadAsArray() instead of ReadRaster()
+
+    gdaltest.importorskip_gdal_array()
     numpy = pytest.importorskip("numpy")
 
     ds = gdal.Translate(
@@ -3028,6 +3021,7 @@ def test_rasterio_writeraster_from_memoryview():
 # Test ReadRaster() in an existing buffer
 
 
+@gdaltest.disable_exceptions()
 def test_rasterio_readraster_in_existing_buffer():
 
     ds = gdal.GetDriverByName("MEM").Create("", 2, 1)
@@ -3064,6 +3058,7 @@ def test_rasterio_readraster_in_existing_buffer():
 # Test ReadBlock() in an existing buffer
 
 
+@gdaltest.disable_exceptions()
 def test_rasterio_readblock_in_existing_buffer():
 
     ds = gdal.GetDriverByName("MEM").Create("", 2, 1)
@@ -3090,6 +3085,7 @@ def test_rasterio_readblock_in_existing_buffer():
 # Test ReadRaster() in an existing buffer and alignment issues
 
 
+@gdaltest.disable_exceptions()
 @pytest.mark.parametrize(
     "datatype",
     [
@@ -3175,6 +3171,8 @@ def test_rasterio_gdal_rasterio_resampling():
 
 
 def test_rasterio_numpy_datatypes_for_xoff():
+
+    gdaltest.importorskip_gdal_array()
     np = pytest.importorskip("numpy")
 
     ds = gdal.Open("data/byte.tif")
@@ -3406,3 +3404,18 @@ def test_rasterio_overview_selection():
         assert ds.GetRasterBand(1).ReadRaster(0, 0, 100, 100, 29, 29)[0] == 2
         assert ds.GetRasterBand(1).ReadRaster(0, 0, 100, 100, 25, 25)[0] == 3
         assert ds.GetRasterBand(1).ReadRaster(0, 0, 100, 100, 24, 24)[0] == 3
+
+
+###############################################################################
+# Check robustness to GDT_Unknown
+
+
+def test_rasterio_gdt_unknown():
+
+    with gdal.GetDriverByName("MEM").Create("", 1, 1) as ds:
+        # Caught at the SWIG level
+        with pytest.raises(Exception, match="Illegal value for data type"):
+            ds.ReadRaster(buf_type=gdal.GDT_Unknown)
+        # Caught at the SWIG level
+        with pytest.raises(Exception, match="Illegal value for data type"):
+            ds.GetRasterBand(1).ReadRaster(buf_type=gdal.GDT_Unknown)
