@@ -1,6 +1,5 @@
 #!/usr/bin/env pytest
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Test driver metadata
@@ -9,23 +8,7 @@
 ###############################################################################
 # Copyright (c) 2020, Rene Buffat <buffat at gmail dot com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 
@@ -147,6 +130,7 @@ schema_creationoptionslist_xml = etree.XML(
         <xs:extension base="xs:string">
           <xs:attribute type="xs:string" name="alias" use="optional"/>
           <xs:attribute type="xs:string" name="aliasOf" use="optional"/>
+          <xs:attribute type="xs:string" name="remark" use="optional"/>
         </xs:extension>
       </xs:simpleContent>
     </xs:complexType>
@@ -392,6 +376,13 @@ schema_multidim_dimension_creationoptionslist_xml = etree.XML(
 )
 
 
+@pytest.mark.parametrize("driver_name", all_driver_names)
+def test_metadata_has_long_name(driver_name):
+
+    driver = gdal.GetDriverByName(driver_name)
+    assert driver.GetMetadataItem(gdal.DMD_LONGNAME) is not None
+
+
 @pytest.mark.parametrize("driver_name", all_driver_names)
 def test_metadata_dcap_yes(driver_name):
     """Test that the only value of DCAP_ elements is YES"""
@@ -652,3 +643,30 @@ def test_metadata_creation_field_defn_flags(driver_name):
     if flags_str is not None:
         for flag in flags_str.split(" "):
             assert flag in supported_flags
+
+
+@pytest.mark.parametrize("driver_name", ogr_driver_names)
+def test_metadata_update_items(driver_name):
+    """Test if DMD_UPDATE_ITEMS metadataitem returns valid flags"""
+
+    supported_flags = {
+        "GeoTransform",
+        "SRS",
+        "GCPs",
+        "NoData",
+        "ColorInterpretation",
+        "RasterValues",
+        "DatasetMetadata",
+        "BandMetadata",
+        "Features",
+        "LayerMetadata",
+    }
+
+    driver = gdal.GetDriverByName(driver_name)
+    flags_str = driver.GetMetadataItem(gdal.DMD_UPDATE_ITEMS)
+    if flags_str is not None:
+        assert driver.GetMetadataItem(gdal.DCAP_UPDATE)
+        for flag in flags_str.split(" "):
+            assert flag in supported_flags
+    else:
+        assert not driver.GetMetadataItem(gdal.DCAP_UPDATE)
