@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 ###############################################################################
-# $Id$
 #
 # Project:  GDAL/OGR Test Suite
 # Purpose:  Python Library supporting GDAL/OGR Test Suite
@@ -10,26 +9,11 @@
 # Copyright (c) 2003, Frank Warmerdam <warmerdam@pobox.com>
 # Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
 #
-# Permission is hereby granted, free of charge, to any person obtaining a
-# copy of this software and associated documentation files (the "Software"),
-# to deal in the Software without restriction, including without limitation
-# the rights to use, copy, modify, merge, publish, distribute, sublicense,
-# and/or sell copies of the Software, and to permit persons to whom the
-# Software is furnished to do so, subject to the following conditions:
-#
-# The above copyright notice and this permission notice shall be included
-# in all copies or substantial portions of the Software.
-#
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
-# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
-# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
-# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
-# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
-# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
-# DEALINGS IN THE SOFTWARE.
+# SPDX-License-Identifier: MIT
 ###############################################################################
 
 import contextlib
+import functools
 import io
 import json
 import math
@@ -49,20 +33,18 @@ from threading import Thread
 
 import pytest
 
-from osgeo import gdal, ogr, osr
+from osgeo import gdal, osr
 
 jp2kak_drv = None
 jpeg2000_drv = None
 jp2ecw_drv = None
 jp2mrsid_drv = None
 jp2openjpeg_drv = None
-jp2lura_drv = None
 jp2kak_drv_unregistered = False
 jpeg2000_drv_unregistered = False
 jp2ecw_drv_unregistered = False
 jp2mrsid_drv_unregistered = False
 jp2openjpeg_drv_unregistered = False
-jp2lura_drv_unregistered = False
 
 ###############################################################################
 
@@ -90,7 +72,7 @@ def testCreateCopyInterruptCallback(pct, message, user_data):
 ###############################################################################
 
 
-class GDALTest(object):
+class GDALTest:
     def __init__(
         self,
         drivername,
@@ -237,6 +219,14 @@ class GDALTest(object):
                             drv_name.lower() == "biggif"
                             and self.drivername.lower() == "gif"
                         )
+                        or (
+                            drv_name.lower() == "gtiff"
+                            and self.drivername.lower() == "libertiff"
+                        )
+                        or (
+                            drv_name.lower() == "libertiff"
+                            and self.drivername.lower() == "gtiff"
+                        )
                     ):
                         drivers += [drv_name]
                 other_ds = None
@@ -1280,8 +1270,8 @@ def compare_ds(ds1, ds2, xoff=0, yoff=0, width=0, height=0, verbose=1):
 
 
 def deregister_all_jpeg2000_drivers_but(name_of_driver_to_keep):
-    global jp2kak_drv, jpeg2000_drv, jp2ecw_drv, jp2mrsid_drv, jp2openjpeg_drv, jp2lura_drv
-    global jp2kak_drv_unregistered, jpeg2000_drv_unregistered, jp2ecw_drv_unregistered, jp2mrsid_drv_unregistered, jp2openjpeg_drv_unregistered, jp2lura_drv_unregistered
+    global jp2kak_drv, jpeg2000_drv, jp2ecw_drv, jp2mrsid_drv, jp2openjpeg_drv
+    global jp2kak_drv_unregistered, jpeg2000_drv_unregistered, jp2ecw_drv_unregistered, jp2mrsid_drv_unregistered, jp2openjpeg_drv_unregistered
 
     # Deregister other potential conflicting JPEG2000 drivers that will
     # be re-registered in the cleanup
@@ -1315,12 +1305,6 @@ def deregister_all_jpeg2000_drivers_but(name_of_driver_to_keep):
         jp2openjpeg_drv.Deregister()
         jp2openjpeg_drv_unregistered = True
 
-    jp2lura_drv = gdal.GetDriverByName("JP2Lura")
-    if name_of_driver_to_keep != "JP2Lura" and jp2lura_drv:
-        gdal.Debug("gdaltest.", "Deregistering JP2Lura")
-        jp2lura_drv.Deregister()
-        jp2lura_drv_unregistered = True
-
     return True
 
 
@@ -1330,8 +1314,8 @@ def deregister_all_jpeg2000_drivers_but(name_of_driver_to_keep):
 
 
 def reregister_all_jpeg2000_drivers():
-    global jp2kak_drv, jpeg2000_drv, jp2ecw_drv, jp2mrsid_drv, jp2openjpeg_drv, jp2lura_drv
-    global jp2kak_drv_unregistered, jpeg2000_drv_unregistered, jp2ecw_drv_unregistered, jp2mrsid_drv_unregistered, jp2openjpeg_drv_unregistered, jp2lura_drv_unregistered
+    global jp2kak_drv, jpeg2000_drv, jp2ecw_drv, jp2mrsid_drv, jp2openjpeg_drv
+    global jp2kak_drv_unregistered, jpeg2000_drv_unregistered, jp2ecw_drv_unregistered, jp2mrsid_drv_unregistered, jp2openjpeg_drv_unregistered
 
     if jp2kak_drv_unregistered:
         jp2kak_drv.Register()
@@ -1358,11 +1342,6 @@ def reregister_all_jpeg2000_drivers():
         jp2openjpeg_drv_unregistered = False
         gdal.Debug("gdaltest", "Registering JP2OpenJPEG")
 
-    if jp2lura_drv_unregistered:
-        jp2lura_drv.Register()
-        jp2lura_drv_unregistered = False
-        gdal.Debug("gdaltest", "Registering JP2Lura")
-
     return True
 
 
@@ -1914,6 +1893,8 @@ def gdalurlopen(url, timeout=10):
 
         urllib.request.install_opener(opener)
 
+    import http.client
+
     try:
         handle = urllib.request.urlopen(url)
         socket.setdefaulttimeout(old_timeout)
@@ -1934,6 +1915,10 @@ def gdalurlopen(url, timeout=10):
         print(f"HTTP service for {url} timed out")
         socket.setdefaulttimeout(old_timeout)
         return None
+    except http.client.RemoteDisconnected as e:
+        print(f"HTTP service for {url} is not available: RemoteDisconnected : {e}")
+        socket.setdefaulttimeout(old_timeout)
+        return None
 
 
 def runexternal(
@@ -1982,13 +1967,20 @@ def _read_in_thread(f, q):
     f.close()
 
 
-def runexternal_out_and_err(cmd, check_memleak=True, encoding="ascii"):
+def runexternal_out_and_err(
+    cmd, check_memleak=True, encoding="ascii", stdin=None, close_stdin=False
+):
     # pylint: disable=unused-argument
     if sys.platform == "win32":
         command = cmd
     else:
         command = shlex.split(cmd)
-    p = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    p = subprocess.Popen(
+        command, stdin=stdin, stdout=subprocess.PIPE, stderr=subprocess.PIPE
+    )
+
+    if close_stdin:
+        p.stdin.close()
 
     if p.stdout is not None:
         q_stdout = Queue()
@@ -2097,9 +2089,6 @@ def reopen(ds, update=False, open_options=None):
 
     ds.Close()
 
-    if isinstance(ds, ogr.DataSource) and open_options is None:
-        return ogr.Open(ds_loc, update)
-
     flags = 0
     if update:
         flags = gdal.OF_UPDATE
@@ -2113,3 +2102,59 @@ def reopen(ds, update=False, open_options=None):
         allowed_drivers=[ds_drv.GetDescription()],
         open_options=open_options,
     )
+
+
+def vsi_open(path, mode="r"):
+    return gdal.VSIFile(path, mode)
+
+
+def vrt_has_open_support():
+    drv = gdal.GetDriverByName("VRT")
+    return drv is not None and drv.GetMetadataItem(gdal.DMD_OPENOPTIONLIST) is not None
+
+
+###############################################################################
+# Check that an error or warning is raised
+
+
+@contextlib.contextmanager
+def error_raised(type, match=""):
+
+    err_levels = {
+        gdal.CE_Debug: "CE_Debug",
+        gdal.CE_Failure: "CE_Failure",
+        gdal.CE_Fatal: "CE_Fatal",
+        gdal.CE_None: "CE_None",
+        gdal.CE_Warning: "CE_Warning",
+    }
+
+    errors = []
+
+    def handler(lvl, no, msg):
+        errors.append({"level": lvl, "number": no, "message": msg})
+
+    with error_handler(handler):
+        yield
+
+    assert any(
+        [err["level"] == type and match in err["message"] for err in errors]
+    ), f'Did not receive an error of type {err_levels[type]} matching "{match}". Received: {[(err["level"], err["message"]) for err in errors]}'
+
+
+###############################################################################
+# Check VRT capabilities
+
+
+@functools.lru_cache()
+def gdal_has_vrt_expression_dialect(dialect):
+    return dialect in gdal.GetDriverByName("VRT").GetMetadataItem("ExpressionDialects")
+
+
+###############################################################################
+
+
+def importorskip_gdal_array():
+    pytest_version = [int(x) for x in pytest.__version__.split(".")]
+    if pytest_version >= [8, 2, 0]:
+        return pytest.importorskip("osgeo.gdal_array", exc_type=ImportError)
+    return pytest.importorskip("osgeo.gdal_array")
